#!/bin/sh
#####################################################################
# Copyright   : Copyright (C) 2007 NTT
#/**
# * @file    promote.sh
# * @brief   FMS連携機能 - ACT化コマンド(promote)
# *
# * SBY状態のPREXシステムをACT状態に遷移させる。
# *
# *   詳細は以下のとおり。
# *     -# 設定ファイル(WALWriter.conf)より、パラメータを取得する。取得するパラメータを以下に示す。
# *       - prex_msgkey (メッセージキューのキー値)
# *       - acthahost (Heartbeat LANを経由する相手ノード宛のIPアドレス)
# *       - actwalhost (データ転送LANを経由する相手ノード宛のIPアドレス)
# *       - sbywaldir (ローカルサーバのWALセグメント格納ディレクトリ)
# *       - sbyarchivedir (ローカルサーバのアーカイブログ格納ディレクトリ)
# *     -# 状態遷移機能のSBY状況確認を呼び出し、PREXの状態がSBYであることを確認する。
# *     -# SBY監視抑止ファイルおよびACT監視抑止ファイルを「監視抑止状態」に更新する。
# *     -# 起動オプションの値が「通常起動」の場合、4～5の処理を行ないACT化可否を判定する。
# *     -# Heartbeat LAN経由でリモートサーバのACT状況確認を呼び出し、リモートサーバのPREXがACTでないことを確認する。ACTであった場合は、戻り値にACT化失敗を返却する。Heartbeat LAN経由でのリモートサーバへのアクセスに失敗した場合は、データ転送LAN経由で同処理を行なう。
# *     -# Heartbeat LAN経由でリモートサーバのフェールオーバー抑止ファイルからステータスを取得する。ステータスが「1」(ACT化抑止)であった場合は、戻り値にACT化失敗を返却する。Heartbeat LAN経由でのリモートサーバへのアクセスに失敗した場合は、データ転送LAN経由で同処理を行なう。
# *       - 4または5の処理で、Heartbeat LAN経由、データ転送LAN経由の両者でリモートサーバへのアクセスに失敗した場合は、マシン障害時のフェールオーバー動作オプションを確認する。OFF(フェールオーバー抑止)の場合は、戻り値にACT化失敗を返却する。
# *     -# 状態遷移機能のACT化要求を呼び出し、PREXをACTに遷移させる。
# *     -# ACT監視抑止ファイルを「監視状態」に更新する。
# *
# * @param[in]   mode_starup 起動モード
# * @param[in]   mode_fo     マシン障害によるフェールオーバー時の動作オプション
# * @retval      RET_OK(0)   ACT化成功
# * @retval      RET_NG(1)   ACT化失敗
# *
# * @version 1.0.0
# * @date    08/01/2007
# * @since   1.0
# */

# DEFINE値取り込み
. ${HOME}/prex/bin/prexdefine.sh

##
## 定数
##
DEFAULT_IFS="${IFS}" # IFS(デフォルト値)

#/**
# * @fn      function reset_monitor_control()
# * @brief   監視抑止を元の状態に復元する。
# *
# * 当コマンドでエラーが発生した場合に呼ばれる。
# * 監視抑止を元の状態に戻す。戻す状態は以下の通り。
# *  - SBY監視：「監視状態」- SBY監視抑止ファイルを「監視状態」に更新
# *  - ACT監視：「not running」- ACT監視抑止ファイルを削除
# *
# *   詳細は以下のとおり。
# *     -# SBY監視抑止ファイルを「監視状態」に更新する。
# *     -# ACT監視抑止ファイルを削除する。
# *
# * @retval    RET_OK(0) 正常終了
# * @retval    RET_NG(1) 異常終了
# *
# * @version 1.0.0
# * @date    08/02/2007
# * @since   1.0
# */
function reset_monitor_control()
{
	##
	## 監視抑止を設定
	## ・SBY監視：「監視状態」- SBY監視抑止ファイルを「監視状態」に更新
	## ・ACT監視：「not running」- ACT監視抑止ファイルを削除
	##
	${PREX_BIN}/exclusion.sh "${SBY_LOCK_COMMANDID}" "${FILE_SET}" "${MONITOR_TYPEID_UNLOCK}" && \
	${PREX_BIN}/exclusion.sh "${ACT_LOCK_COMMANDID}" "${FILE_RM}"

	if [ ${?} -ne ${RET_OK} ] ; then
		${PREX_BIN}/prexsyslog ${LOG_ERR} ${MSGID_FMSIF_ERROREXCLUSION} ${0} ${LINENO}
		return ${RET_NG}
	fi
	
	return ${RET_OK}
}

#######################################
## メインルート
##

[ "${DEBUG_LOG}" == 'ON' ] && ${PREX_BIN}/prexsyslog ${LOG_DEBUG} ${MSGID_FMSIF_DEBUG} ${0} ${LINENO} "[start] promote.sh (mode=${1}, enable_fo=${2})"

##
## 変数宣言
##
mode_startup="${1}" # 起動モード
mode_fo="${2}"      # マシン障害によるフェールオーバー時の動作オプション
msgkey=             # メッセージキューのキー値
param_list=()       # 設定ファイルの設定値一覧

##
## 設定ファイル(WALWriter.conf)より、以下のパラメータを取得
## ・acthahost (Heartbeat LANを経由する相手ノード宛のIPアドレス)
## ・actwalhost (データ転送LANを経由する相手ノード宛のIPアドレス)
## ・sbywaldir (ローカルサーバのWALセグメント格納ディレクトリ)
## ・sbyarchivedir (ローカルサーバのアーカイブログ格納ディレクトリ)
##
IFS='
'
param_list=(`${PREX_BIN}/ReadConfig "${wwconffile}"`)

if [ ${?} -ne ${RET_OK} ] ; then
	${PREX_BIN}/prexsyslog ${LOG_ERR} ${MSGID_FMSIF_FAILEDTOREADCONFIG} ${0} ${LINENO}
	exit ${RET_NG}
fi

for param in "${param_list[@]}"
do
    eval "${param}"
done
IFS="${DEFAULT_IFS}"

##
## 設定ファイル(WALWriter.conf)より、メッセージキューのキー値を取得
##
msgkey="`${PREX_BIN}/getmsgkey.sh`"

if [ ${?} -ne ${RET_OK} ] ; then
	${PREX_BIN}/prexsyslog ${LOG_ERR} ${MSGID_FMSIF_ERRORGETMSGKEY} ${0} ${LINENO}
	exit ${RET_NG}
fi

##
## 状態遷移機能のSBY状況確認を呼び出し、PREXの状態がSBYであることを確認
##
${PREX_BIN}/sbymonitor.sh ${msgkey} -N

if [ ${?} -ne ${RET_OK} ] ; then
	${PREX_BIN}/prexsyslog ${LOG_ERR} ${MSGID_FMSIF_ERRORSTATENOTMATCH} ${0} ${LINENO}
	exit ${RET_NG}
fi

##
## 監視抑止を設定
## ・SBY監視：「監視抑止状態」- SBY監視抑止ファイルを「監視抑止状態」に更新
## ・ACT監視：「監視抑止状態」- ACT監視抑止ファイルを「監視抑止状態」に更新
##
${PREX_BIN}/exclusion.sh "${SBY_LOCK_COMMANDID}" "${FILE_SET}" "${MONITOR_TYPEID_LOCK}" && \
${PREX_BIN}/exclusion.sh "${ACT_LOCK_COMMANDID}" "${FILE_SET}" "${MONITOR_TYPEID_LOCK}"

if [ ${?} -ne ${RET_OK} ] ; then
	${PREX_BIN}/prexsyslog ${LOG_ERR} ${MSGID_FMSIF_ERROREXCLUSION} ${0} ${LINENO}
	exit ${RET_NG}
fi

##
## 起動モードが「NC」(通常起動)の場合、ACT化可否を判定
##
if [ "${mode_startup}" = 'NC' ] ; then
	check_enable_toact.sh ${msgkey} "${acthahost}" "${actwalhost}" "${enabletoactretrycount}" "${enabletoactretryinterval}" "${mode_fo}"

	if [ ${?} -ne ${RET_OK} ] ; then
		${PREX_BIN}/prexsyslog ${LOG_WARNING} ${MSGID_FMSIF_FOCONTROLDONE} ${0} ${LINENO}
		reset_monitor_control
		exit ${RET_NG}
	fi
fi

##
## 状態遷移機能のACT化要求を実行
## ・WALWriterを起動
## ・postmasterを起動
## ・メッセージキューを作成
##
${PREX_BIN}/toact.sh ${msgkey} "${sbywaldir}" "${sbyarchivedir}"

if [ ${?} -ne ${RET_OK} ] ; then
	${PREX_BIN}/prexsyslog ${LOG_ERR} ${MSGID_FMSIF_ERRORTOACT} ${0} ${LINENO}
	reset_monitor_control
	exit ${RET_NG}
fi

##
## 監視抑止を設定
## ・ACT監視：「監視状態」- ACT監視抑止ファイルを「監視状態」に更新
##
${PREX_BIN}/exclusion.sh "${ACT_LOCK_COMMANDID}" "${FILE_SET}" "${MONITOR_TYPEID_UNLOCK}"

if [ ${?} -ne ${RET_OK} ] ; then
	${PREX_BIN}/prexsyslog ${LOG_ERR} ${MSGID_FMSIF_ERROREXCLUSION} ${0} ${LINENO}
	exit ${RET_NG}
fi

[ "${DEBUG_LOG}" == 'ON' ] && ${PREX_BIN}/prexsyslog ${LOG_DEBUG} ${MSGID_FMSIF_DEBUG} ${0} ${LINENO} "[end]   promote.sh (ret=${RET_OK})"

exit ${RET_OK}
